// -*- mode:c++ -*-
// OpenMP+CUDA scheme for the problem imagen processing (PIP)
// Spanich Parallel Programming Contest 2012


#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <omp.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>

#include <cutil_inline.h>


void Image_Smoothing_by_Filtering(unsigned char *Image, unsigned int N, int Rep);


////////////////////////////////////////////////////////////////
////                  Auxiliar C procedures                /////
////////////////////////////////////////////////////////////////

void initialize(unsigned char *Matrix, unsigned int Size)
{
  unsigned int
     i, j;

  // Values between 0 and 255 are generated randomly
  for(i=1; i<=Size; i++)
     for(j=1; j<=Size; j++)
        Matrix[i*(Size+2)+j] =(unsigned char)(256.0*rand() / RAND_MAX);
}

void escribir(unsigned char *Matrix, unsigned int Size)
{
  unsigned int
     i, j;

  for (i=0; i<=(Size+1); i++)
  {
    for (j=0; j<=(Size+1); j++)
      printf("%3u ", Matrix[i*(Size+2)+j]);
    printf("\n");
  }
  printf("\n");
}

long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}


static void write_checksum(unsigned char *Matrix, unsigned int Size) {
  unsigned long long sum = 0;
  for (unsigned int i = 1; i <= Size; ++i) {
    for (unsigned int j = 1; j <= Size; ++j) {
      sum += Matrix[i * (Size + 2) + j];
    }
  }
  printf("%llu\n", sum);
}

////////////////////////////////////////////////////////////////////////////////
////////////////                   T H E    M A I N              ///////////////
////////////////////////////////////////////////////////////////////////////////

int main(int argc,char *argv[]) {
  int
     Rep, cuantos, semilla, i, nodo=0, np=1;

  unsigned int
     N;

  long long 
     ti, tf, tt=0.0;

  unsigned char
     *Matrix;

  FILE *stats_file    = fopen("stats", "w");

  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(20);  /* time limit */


#ifdef DEBUG
   int iam =0, np = 1;
   #pragma omp parallel private(iam, np)
   {
      np  = omp_get_num_threads();
      iam = omp_get_thread_num();
      printf("Hello from thread %d out of %d \n", iam, np);
   }
#endif


  // The number of test cases is read
  scanf("%d",&cuantos);

  for(i=1;i<=cuantos;i++)
  {
     scanf("%u",&N);           // The first argument is the matrix size
     scanf("%d",&Rep);         // The second is the number of times per test case
     scanf("%d",&semilla);     // This is the seed for random generation

     // Space for the matrix
     // Remember: the image has a white picture frame (first row and column, last row and column) which should not be processed
     // but it can be used to calculate the elements of A. In other words, A is a NxN matrix stored in a (N+2)x(N+2) area.
     Matrix = (unsigned char *)calloc((N+2)*(N+2), sizeof(unsigned char));

     srand(semilla);
     initialize(Matrix, N);

#ifdef DEBUG
      // While debugging the matrix is written
      escribir(Matrix,N);
#endif

     ti=mseconds(); 
        Image_Smoothing_by_Filtering(Matrix, N, Rep);
     tf=mseconds(); 

     // The time of the first problem is not considered
     if(i!=1) tt+=tf-ti;

#ifdef TIME
      // To write the execution time of each running
      printf(" ti=%Ld \t tf=%Ld  \t tf-ti=%Ld \n ",ti,tf,tf-ti);
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif

     // Only the solution of the last value of i is written
     if (i==cuantos) write_checksum(Matrix, N);

     free(Matrix);
  }//for cuantos
  
  fprintf(stats_file, "%Ld\n", tt);
  fclose (stats_file);

  return 0;
}
